using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Windows;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using System.Windows.Threading;

namespace XamPivotGridXampl
{
    public class BucketRiskFlatDataSourceFactory
    {
        public static BucketRiskFlatDataSource Create()
        {
            return Create(BucketRiskDtoFactory.CreateAll());
        }

        public static BucketRiskFlatDataSource Create(IEnumerable<BucketRiskDto> riskItems)
        {
            var flatDataSource = new BucketRiskFlatDataSource()
            {
                ItemsSource = new ObservableCollection<BucketRiskDto>(riskItems),
                PreserveMembersOrder = false,
            };

            var levelSortDirection = new LevelSortDirection {SortDirection = ListSortDirection.Ascending, LevelUniqueName = "[Tenor].[Tenor]"};
            flatDataSource.LevelDefaultSortDirections.Add(levelSortDirection);

            foreach (var hierarchyDescriptor in GetDefaultHierarchyLevelDescriptor())
            {
                flatDataSource.AddHierarchyDescriptor(hierarchyDescriptor);
            }

            // I have this dispatcher here to delay this code until the XamPivotGrid ItemsSource has been set to the
            // FlatDataSource.  You probably don't want to do it this way.

            Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() =>
            {
                var measure = flatDataSource.Cube.Measures["Risk"];
                var measureViewModel = flatDataSource.CreateMeasureViewModel(measure);
                flatDataSource.Measures.Add(measureViewModel);

                var rowDimension = flatDataSource.Cube.Dimensions["[Tenor]"];
                var rowHierarchy = rowDimension.Hierarchies["[Tenor].[Tenor]"];
                var rowViewModel = flatDataSource.CreateFilterViewModel(rowHierarchy);
                flatDataSource.Rows.Add(rowViewModel);

                var colDimension = flatDataSource.Cube.Dimensions["[AssetType]"];
                var colHierarchy = colDimension.Hierarchies["[AssetType].[AssetType]"];
                var colViewModel = flatDataSource.CreateFilterViewModel(colHierarchy);
                flatDataSource.Columns.Add(colViewModel);
            }), DispatcherPriority.Background);


            return flatDataSource;
        }

        private static IEnumerable<HierarchyDescriptor> GetDefaultHierarchyLevelDescriptor()
        {
            yield return new HierarchyDescriptor<BucketRiskDto>(p => p.Tenor)
            {
                LevelDescriptors =
                {
                    CreateDescriptor(model => "Tenors", "Tenors"),
                    CreateHierarchyLevelDescriptor<BucketRiskDto>(model => model.Tenor, model => DfAddPeriod(model), "Tenor")
                }
            };

            yield return new HierarchyDescriptor<BucketRiskDto>(model => model.AssetType)
            {
                LevelDescriptors =
                {
                    CreateDescriptor(model => "Assets", "Assets"),
                    CreateDescriptor(model => model.AssetType, "AssetType"),
                },
            };

            yield return new HierarchyDescriptor<BucketRiskDto>(model => model.Curve)
            {
                LevelDescriptors =
                {
                    CreateDescriptor(model => "Curves", "Curves"),
                    CreateDescriptor(model => model.Curve, "Curve"),
                },
            };

            yield return new HierarchyDescriptor<BucketRiskDto>(model => model.Book)
            {
                LevelDescriptors =
                {
                    CreateDescriptor(model => "Books", "Books"),
                    CreateDescriptor(model => model.Book, "Book"),
                },
            };
        }

        private static HierarchyLevelDescriptor CreateDescriptor(Expression<Func<BucketRiskDto, string>> levelExpression, string levelName)
        {
            return new HierarchyLevelDescriptor(levelExpression, levelName);
        }

        private static HierarchyLevelDescriptor CreateHierarchyLevelDescriptor<T>(Expression<Func<T, string>> levelExpression, Expression<Func<T, double>> sortExpression, string levelName)
        {
            return new HierarchyLevelDescriptor(levelExpression, levelName) { OrderByKeyExpression = sortExpression };
        }

        private static double DfAddPeriod(BucketRiskDto model)
        {
            var dateTime = DateTime.Today.AddDays(GetDays(model.Tenor));
            return dateTime.ToOADate();
        }

        private static double GetDays(string tenor)
        {
            switch (tenor)
            {
                case "2D":
                    return 2;
                case "1W":
                    return 7;
                case "3M":
                    return 90;
                case "1Y":
                    return 360;
                case "5Y":
                    return 360*5;
                default:
                    throw new NotImplementedException();
            }
        }
    }
}